#include <fstream.h>
#include "Echo.H"

// This client reads a stringified IOR from a file passed as argument

#define IOR_MAXLEN 1024

static void hello( Echo_ptr echo )
{
  CORBA::String_var src = (const char*) "Hello there!";

  CORBA::String_var dest = echo->echoString( src );

  cout << "I said [" << (char*)src << "]." << endl;
  cout << "The echo replied [" << (char*)dest << "]." << endl;
}


int main( int argc, char** argv )
{
  try
    {
      CORBA::ORB_var orb = CORBA::ORB_init( argc, argv );

      if( argc != 2 )
      {
	cerr << "Please specify server object URI" << endl;
	return 1;
      }

      const char* sior = argv[1];
      
      // Convert stringified IOR to Object Reference
      CORBA::Object_var obj = orb->string_to_object( sior );
      if( CORBA::is_nil( obj ) )
	{
	  cerr << "Invalid object reference in file " << argv[1] << endl;
	  return 1;
	}

      // Narrow object reference to Echo
      Echo_var echoref = Echo::_narrow( obj );
      if( CORBA::is_nil( echoref ) )
	{
	  cerr << "Cannot narrow object reference to type Echo" << endl;
	  return 1;
	}
  
      for( int count=0; count<5; count++ )
	{
	  hello( echoref );
	}

      orb->destroy();
    }
  catch( CORBA::COMM_FAILURE& ex )
    {
      cerr << "Exception COMM_FAILURE" << endl;
    }
  catch( CORBA::SystemException& ex )
    {
      cerr << "CORBA System Exception" << endl;
    }
  catch( CORBA::Exception& ex )
    {
      cerr << "CORBA Exception" << endl;
    }
  catch( ... )
    {
      cerr << "Unexpected exception" << endl;
    }

  return 0;
}
