//
// Example code for implementing IDL interfaces in file Echo.idl
//

#include <fstream.h>
#include <Echo.H>

//
// Example class implementing IDL interface Echo
//
class Echo_i: public POA_Echo,
	      public PortableServer::RefCountServantBase
{
private:
  // Make sure all instances are built on the heap by making the
  // destructor non-public
  virtual ~Echo_i();

public:
  // standard constructor
  Echo_i();
  //virtual ~Echo_i();

  // methods corresponding to defined IDL attributes and operations
  char* echoString(const char* msg);
};

//
// Example implementational code for IDL interface Echo
//
Echo_i::Echo_i()
{
  // add extra constructor code here
}

Echo_i::~Echo_i()
{
  // add extra destructor code here
}

//   Methods corresponding to IDL attributes and operations
char* Echo_i::echoString(const char* msg)
{
  cout << "Echo sent for string [" << msg << "]" << endl;
  return CORBA::string_dup( msg );
}

int main(int argc, char** argv)
{
  try {
    // Initialise the ORB.
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

    // Obtain a reference to the omniINS POA.
    CORBA::Object_var obj = orb->resolve_initial_references("omniINSPOA");
    if( CORBA::is_nil( obj ) )
      {
	cerr << "Cannot get hold of omniINSPOA" << endl;
	return 1;
      }

    PortableServer::POA_var insPoa = PortableServer::POA::_narrow(obj);
    if( CORBA::is_nil( insPoa ) )
      {
	cerr << "Cannot narrow omniINSPOA" << endl;
	return 1;
      }
    
    // We allocate the objects on the heap.  Since these are reference
    // counted objects, they will be deleted by the POA when they are no
    // longer needed.
    Echo_i* myEcho_i = new Echo_i();

    // Create unique object key for IOR
    PortableServer::ObjectId_var echoOid =
      PortableServer::string_to_ObjectId( "TheEcho" );
    
    insPoa->activate_object_with_id( echoOid, myEcho_i );
    cout << "Echo object activated with key [TheEcho]" << endl;

    // Obtain a POAManager, and tell the POA to start accepting
    // requests on its objects.
    PortableServer::POAManager_var pman = insPoa->the_POAManager();
    pman->activate();

    cout << "Echo Server ready" << endl;
    orb->run();
    orb->destroy();
  }
  catch(CORBA::SystemException&) {
    cerr << "Caught CORBA::SystemException." << endl;
  }
  catch(CORBA::Exception&) {
    cerr << "Caught CORBA::Exception." << endl;
  }
  catch(omniORB::fatalException& fe) {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << "  file: " << fe.file() << endl;
    cerr << "  line: " << fe.line() << endl;
    cerr << "  mesg: " << fe.errmsg() << endl;
  }
  catch(...) {
    cerr << "Caught unknown exception." << endl;
  }

  return 0;
}

